exception ALCException of string

type formula =
  | TRUE
  | FALSE
  | AP of string
  | NOT of formula
  | EX of string * bool * formula
  | AX of string * bool * formula
  | MIN of int * string * bool * formula
  | MAX of int * string * bool * formula
  | EQU of formula * formula
  | IMP of formula * formula
  | OR of formula * formula
  | AND of formula * formula

type definition =
  | PRIMITIVE of string * formula
  | NECANDSUFF of string * formula

val numberOfTypes : int
val getTypeFormula : formula -> int
val generateCompare : int list -> (formula -> formula -> int)
val aRanking : int list

val exportFormula : formula -> string
val exportQuery : formula list -> formula list -> definition list -> string
val importFormula : string -> formula
val importQuery : string -> (formula list) * (formula list) * definition list
val sizeFormula : formula -> int

val substFormula : (string * formula) list -> formula -> formula

val nnfNeg : formula -> formula
val nnf : formula -> formula

val containsInverse : formula -> bool

val simplifyFormula : formula -> formula

val dest_ap : formula -> string
val dest_not : formula -> formula
val dest_ex : formula -> string * bool * formula
val dest_ax : formula -> string * bool * formula
val dest_min : formula -> int * string * bool * formula
val dest_max : formula -> int * string * bool * formula
val dest_equ : formula -> formula * formula
val dest_imp : formula -> formula * formula
val dest_or : formula -> formula * formula
val dest_and : formula -> formula * formula

val is_true : formula -> bool
val is_false : formula -> bool
val is_ap : formula -> bool
val is_not : formula -> bool
val is_ex : formula -> bool
val is_ax : formula -> bool
val is_min : formula -> bool
val is_max : formula -> bool
val is_equ : formula -> bool
val is_imp : formula -> bool
val is_or : formula -> bool
val is_and : formula -> bool
        
val const_true : formula
val const_false : formula
val const_ap : string -> formula
val const_not : formula -> formula
val const_ex : string -> bool -> formula -> formula
val const_ax : string -> bool -> formula -> formula
val const_min : int -> string -> bool -> formula -> formula
val const_max : int -> string -> bool -> formula -> formula
val const_equ : formula -> formula -> formula
val const_imp : formula -> formula -> formula
val const_or : formula -> formula -> formula
val const_and : formula -> formula -> formula

val detClosure : (formula -> formula -> int) -> formula -> formula list

type hcFormula = (hcFormula_node, formula) HashConsing.hash_consed
and hcFormula_node =
  | HCTRUE
  | HCFALSE
  | HCAP of string
  | HCNOT of string
  | HCEX of string * bool * hcFormula
  | HCAX of string * bool * hcFormula
  | HCOR of hcFormula * hcFormula
  | HCAND of hcFormula * hcFormula

module HcFormula : (HashConsing.S with type nde = hcFormula_node and type fml = formula)

module HcFHt : (Hashtbl.S with type key = hcFormula)

val hc_formula : HcFormula.t -> formula -> hcFormula
val detClosureHc : HcFormula.t -> hcFormula -> hcFormula list

val exportFormulaAlcTableau : formula -> string
val createFactTBox : string -> formula list -> formula list -> definition list -> unit
val createWill : string -> formula -> unit
val createOWL : string -> formula list -> formula list -> definition list -> unit
val importTancs2000 : string -> (formula list) * (formula list) * definition list
